/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.data.Historie;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.ibv.uda.uda.data.SkriptZustand;
import de.bsvrz.ibv.uda.uda.data.UDA;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation eines Uda-Skript-Objekts innerhalb einer Client-Anwendung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ClientSkript extends Skript implements ClientReceiverInterface {

	/**
	 * das Modul, in dem das Skript verwaltet wird.
	 */
	private final ClientModul modul;

	/**
	 * des Datenverteiler-Systemobjekt, das das Skript in der Konfiguration des
	 * Datenverteilers repräsentiert.
	 */
	private final SystemObject skriptObjekt;

	/**
	 * die Datenbeschreibung für den Empfang des Istzustandes der definierenden
	 * Daten des Skripts.
	 */
	private DataDescription istDesc;

	/**
	 * die Datenbeschreibung zum Versand von Vorgabedaten für die Definition des
	 * Skripts.
	 */
	private DataDescription vorgabeDesc;

	/**
	 * Marker, der die Gültigkeit der Skriptdaten anzeigt.
	 */
	private boolean valid;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt für das Modul und das übergebene Systemobjekt ein
	 * Skriptobjekt.
	 *
	 * @param modul
	 *            das UdaModul
	 * @param obj
	 *            das Systemobjekt
	 */
	ClientSkript(final ClientModul modul, final SystemObject obj) {
		super(obj.getNameOrPidOrId());
		this.modul = modul;
		this.skriptObjekt = obj;
		initialisiereVerbindungen();
	}

	/**
	 * aktualisiert die Daten des Skriptobjekts mit dem vom Datenverteiler
	 * empfangenen Istwert.
	 *
	 * @param data
	 *            die neuen Daten
	 */
	public final void aktualisiereDaten(final Data data) {
		setSkriptName(data.getTextValue(UDA.SKRIPT_ATT_NAME).getText());
		setSkriptBeschreibung(data.getTextValue(UDA.SKRIPT_ATT_BESCHREIBUNG).getText());
		setErstelltAm(data.getTimeValue(UDA.SKRIPT_ATT_ERSTELLT_AM).getMillis());
		setErstelltDurch(data.getTextValue(UDA.SKRIPT_ATT_ERSTELLT_DURCH).getText());

		clearHistorie();
		final Data.Array historieArray = data.getArray(UDA.SKRIPT_ATT_VERSIONSHISTORIE);
		for (int idx = 0; idx < historieArray.getLength(); idx++) {
			final Data historieData = historieArray.getItem(idx);
			final long zeitPunkt = historieData.getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).getMillis();
			final String name = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_NAME).getText();
			final String grund = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).getText();

			addHistorie(new Historie(zeitPunkt, name, grund));
		}

		setZustand(SkriptZustand.getZustand(data.getUnscaledValue(UDA.SKRIPT_ATT_ZUSTAND).intValue()));
		clearSourceCode();
		final Data.TextArray srcCode = data.getTextArray(UDA.SKRIPT_ATT_QUELLTEXT);
		for (int idx = 0; idx < srcCode.getLength(); idx++) {
			addSourceLine(srcCode.getText(idx));
		}
		valid = true;
	}

	/**
	 * gibt das betreffende Skript zur Ausführung frei.
	 *
	 * @param benutzer
	 *            der Benutzer
	 * @param grund
	 *            der Grund für die Freigabe
	 * @throws ClientException
	 *             die Freigabe konnte nicht ausgeführt werden
	 */
	public void freigeben(final String benutzer, final String grund) throws ClientException {

		setZustand(SkriptZustand.FREIGEGEBEN);

		String freigabeText = grund;
		if ((freigabeText == null) || (freigabeText.length() <= 0)) {
			freigabeText = "Skript zur Ausführung freigegeben";
		}

		save(benutzer, freigabeText);
	}

	/**
	 * erzeugt und füllt einen Datensatz zur Vorgabe neuer definierender Daten
	 * für das Skript.
	 *
	 * @param benutzer
	 *            der Name des Benutzers, der die Änderung durchführt
	 * @param grund
	 *            der Grund für die Änderung
	 * @return einen Datensatz für die Beschreibung eines Skripts
	 */
	private Data fuelleDatensatz(final String benutzer, final String grund) {
		final Data data = modul.getVerbindung().createData(istDesc.getAttributeGroup());
		data.getTextValue(UDA.SKRIPT_ATT_NAME).setText(getName());
		data.getTextValue(UDA.SKRIPT_ATT_BESCHREIBUNG).setText(getBeschreibung());
		data.getTimeValue(UDA.SKRIPT_ATT_ERSTELLT_AM).setMillis(getErstellungsDatum());
		data.getTextValue(UDA.SKRIPT_ATT_ERSTELLT_DURCH).setText(getErsteller());

		final Data.Array historie = data.getArray(UDA.SKRIPT_ATT_VERSIONSHISTORIE);
		historie.setLength(1);
		historie.getItem(0).getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).setMillis(System.currentTimeMillis());
		historie.getItem(0).getTextValue(UDA.ATL_HISTORIE_ATT_NAME).setText(benutzer);
		historie.getItem(0).getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).setText(grund);

		data.getUnscaledValue(UDA.SKRIPT_ATT_ZUSTAND).set(getZustand().getCode());
		final Data.Array code = data.getArray(UDA.SKRIPT_ATT_QUELLTEXT);
		code.setLength(getSourceCode().size());
		int offset = 0;
		for (final String line : getSourceCode()) {
			code.getTextValue(offset++).setText(line);
		}
		return data;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.Skript#getDavObjekt()
	 */
	@Override
	public SystemObject getDavObjekt() {
		return skriptObjekt;
	}

	/**
	 * liefert das Modul, dem das Skript zugeordnet ist.
	 *
	 * @return das Modul
	 */
	public ClientModul getModul() {
		return modul;
	}

	/**
	 * liefert den Quelltext des Skripts als eine Zeichenkette.
	 *
	 * @return den Quelltext
	 */
	@Override
	public String getSource() {
		final StringBuffer buffer = new StringBuffer();
		for (final String line : getSourceCode()) {
			buffer.append(line);
			buffer.append(Constants.NL);
		}
		return buffer.toString();
	}

	/**
	 * initialisiert die Verbindung zum Datenverteiler.
	 */
	private void initialisiereVerbindungen() {
		final ClientDavInterface dav = modul.getVerbindung();
		final DataModel model = dav.getDataModel();
		istDesc = new DataDescription(model.getAttributeGroup(UDA.SKRIPT_ATG_SKRIPT), model.getAspect(UDA.ASP_IST));
		vorgabeDesc = new DataDescription(model.getAttributeGroup(UDA.SKRIPT_ATG_SKRIPT),
				model.getAspect(UDA.ASP_VORGABE));
		dav.subscribeReceiver(this, skriptObjekt, istDesc, ReceiveOptions.normal(), ReceiverRole.receiver());

		modul.getSender().anmelden(skriptObjekt, vorgabeDesc, false);
	}

	/**
	 * ermittelt, ob die Daten des Objekts gültig sind, d.h. von einer momentan
	 * aktiven Anwendung "Umfassende Datenanalyse" veröffentlicht wurden.
	 *
	 * @return <i>true</i>, wenn die Daten gültig sind
	 */
	public boolean isValid() {
		return valid;
	}

	/**
	 * die Funktion wird ausgeführt, wenn ein Skriptobjekt aus der Menge der
	 * Skripte entfernt wird. Es meldet die Datenverbindungen des Skriptobjekts
	 * vom Datenverteiler ab.
	 */
	void remove() {
		modul.getVerbindung().unsubscribeReceiver(this, skriptObjekt, istDesc);
		modul.getSender().abmelden(skriptObjekt, vorgabeDesc);
	}

	/**
	 * überträgt die aktuellen Daten des Skriptobjekts.<br>
	 * Die Daten werden vollständig übertragen. Eine Ausnahme bilden die
	 * Historieeinträge, hier wird lediglich ein neuer Eintrag aus den
	 * übergebenem Nutzername und dem definierten Grund erzeugt und an die
	 * "Umfassende Datenanalyse" übertragen.
	 *
	 * @param benutzer
	 *            der Name des Benutzers, der die neuen Daten überträgt
	 * @param grund
	 *            der Grund für die Übertragung der neuen Daten
	 * @throws ClientException
	 *             die Daten konnten nicht erfolgreich übertragen werden
	 */
	public void save(final String benutzer, final String grund) throws ClientException {
		if (!isValid()) {
			throw new ClientException("Daten sind potentiell ungültig");
		}
		try {
			valid = false;
			final Data data = fuelleDatensatz(benutzer, grund);
			modul.getSender().warteAufBestaetigung(skriptObjekt, vorgabeDesc, 20000L);
			modul.getVerbindung().sendData(new ResultData(skriptObjekt, vorgabeDesc, System.currentTimeMillis(), data));
		} catch (final DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			Debug.getLogger().error(e.getLocalizedMessage(), e);
			// kann nicht auftreten, da vor dem Senden auf die
			// Anmeldebestätigung gewartet wird
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion verarbeitet die vom Datenverteiler empfangenen Daten für den
	 * Istzustand der das Skript definierenden Informationen.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(final ResultData[] resultListe) {
		for (final ResultData result : resultListe) {
			final Data data = result.getData();
			if (data != null) {
				this.aktualisiereDaten(data);
			} else {
				valid = false;
			}
			synchronized (getModul().getSkriptListenerListe()) {
				for (final SkriptListener listener : getModul().getSkriptListenerListe()) {
					listener.skriptAktualisiert(this);
				}
			}
		}
	}
}
