/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * eine Anweisungsblock mit Fehlerbehandlung.<br>
 * Die VersucheAnweisung hat zwei Anweisungsblöcke, den eigentlichen
 * Anweisungsblock und einen Anweisungsblock, der ausgeführt wird, wenn eine
 * unbehandelte Ausnahme aufgetreten ist.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class VersucheAnweisung extends AbstractAnweisung {

	/**
	 * die "normalen" Anweisungen.
	 */
	private final AnweisungsBlock anweisungen;

	/**
	 * die Anweisungen, beim Auftreten eines unbehandelten Fehlers.
	 */
	private final AnweisungsBlock fehlerBehandlung;

	/**
	 * die Variable, der das Fehlerbahndlungsobjekt zugewiesen wird.
	 */
	private final Variable fehlerVariable;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung definiert ist
	 * @param skript
	 *            der Name des Skripts
	 * @param anweisungen
	 *            die auszuführenden Anweisungen
	 * @param fehlerVariable
	 *            die Variable für die Zuweisung eines Fehlerobjekts
	 * @param fehlerBehandlung
	 *            die Anweisungen zur Behandlung eines Fehlers.
	 */
	public VersucheAnweisung(final int zeile, final String skript, final AnweisungsBlock anweisungen,
			final Variable fehlerVariable, final AnweisungsBlock fehlerBehandlung) {
		super(zeile, skript);
		this.anweisungen = anweisungen;
		this.fehlerVariable = fehlerVariable;
		this.fehlerBehandlung = fehlerBehandlung;
	}

	/**
	 * fürht die Anweisung aus.<br>
	 * Der Anweisungsblock wird ausgeführt. Ist das Ergebnis der Ausführung ein
	 * Fehlerobjekt oder wurde ein Fehlerobjekt aus einer nicht behandelten
	 * Ausnahme erzeugt, wird der Anweisungsblock zur Fehlerbahndlung
	 * ausgeführt.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = NichtWert.NICHTWERT;

		try {
			result = anweisungen.interpret(kontext);
		} catch (final InterpreterException e) {
			result = erzeugeAnweisungsFehler(e);
			if (kontext instanceof SkriptKontext) {
				final SkriptKontext fehlerKontext = new SkriptKontext((SkriptKontext) kontext, true);
				fehlerKontext.set(fehlerVariable.getName(), result);
				result = fehlerBehandlung.interpret(fehlerKontext);
				for (final String key : kontext.getVariablen()) {
					kontext.set(key, fehlerKontext.get(key));
				}
			} else {
				throw e;
			}
		}

		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(anweisungen);
		liste.add(fehlerVariable);
		liste.add(fehlerBehandlung);
		return liste;
	}
}
