/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.DavConnectionListener;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;

/**
 * Allgemeine Klasse zur Realisierung eines Datenverteiler-Senders mit
 * Sendesteuerung.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public final class BestaetigterDavSender implements ClientSenderInterface, DavConnectionListener {

	/** die Menge aller angelegten Sender-Instanzen. */
	private static final Map<ClientDavInterface, BestaetigterDavSender> SENDER_OBJEKTE = new HashMap<>();

	/**
	 * erzeugt und liefert eine Instanz eines Senderobjekts für die übergebene
	 * Datenverteilerverbindung.
	 *
	 * @param connection
	 *            die verwendete Datenverteilerverbindung
	 * @return die Instanz des Senders
	 */
	public static BestaetigterDavSender getSender(final ClientDavInterface connection) {
		BestaetigterDavSender result = BestaetigterDavSender.SENDER_OBJEKTE.get(connection);
		if (result == null) {
			result = new BestaetigterDavSender(connection);
			BestaetigterDavSender.SENDER_OBJEKTE.put(connection, result);
		}

		return result;
	}

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private final ClientDavInterface verbindung;

	/** die Menge der angemeldeten Sender und Quellen. */
	private final Map<SenderAnmeldung, SenderAnmeldung> anmeldungen = new HashMap<>();

	/**
	 * Konstruktor.<br>
	 * Erzeugt einen Datenverteiler-Sender für die übergebene Verbindung.
	 *
	 * @param verbindung
	 *            die Datenverteiler-Verbindung
	 */
	private BestaetigterDavSender(final ClientDavInterface verbindung) {
		this.verbindung = verbindung;
		verbindung.addConnectionListener(this);
	}

	/**
	 * führt eine Abmeldung für die übergebene Kombination aus Systemobjekt und
	 * Datenbschreibung aus.<br>
	 * Für die entsprechende Anmeldung wird der Anmeldungszähler inkrementiert.
	 * Wenn keine Anmeldung mehr vorhanden ist, erfolgt die Abmeldung vom
	 * Datenverteiler und die Anmeldung wird aus der Liste der verwalteten
	 * Anmeldungen entfernt.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param desc
	 *            die Datenbschreibung
	 */
	public void abmelden(final SystemObject objekt, final DataDescription desc) {
		final SenderAnmeldung abmeldung = new SenderAnmeldung(objekt, desc, false);
		final SenderAnmeldung anmeldung = anmeldungen.get(abmeldung);
		if (anmeldung != null) {
			anmeldung.remove();
			if (anmeldung.isQuelle() || (anmeldung.size() <= 0)) {
				verbindung.unsubscribeSender(this, objekt, desc);
				anmeldungen.remove(anmeldung);
			}
		}
	}

	/**
	 * führt eine Anmeldung für die übergebene Kombination aus Systemobjekt und
	 * Datenbschreibung aus.<br>
	 * Wenn für die Kombination noch keine Anmeldung erfolgte, wird eine neue
	 * Anmeldung ausgeführt, ansonsten wird der Zähler für dies Anmeldung
	 * erhöht.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param desc
	 *            die Datenbschreibung
	 * @param quelle
	 *            die Anmeldung soll als Quelle erfolgen
	 */
	public void anmelden(final SystemObject objekt, final DataDescription desc, final boolean quelle) {

		final SenderAnmeldung neueAnmeldung = new SenderAnmeldung(objekt, desc, quelle);
		final SenderAnmeldung anmeldung = anmeldungen.get(neueAnmeldung);

		if (anmeldung == null) {
			try {
				anmeldungen.put(neueAnmeldung, neueAnmeldung);
				if (quelle) {
					verbindung.subscribeSender(this, objekt, desc, SenderRole.source());
				} else {
					verbindung.subscribeSender(this, objekt, desc, SenderRole.sender());
				}
			} catch (final OneSubscriptionPerSendData e) {
				throw new Fehler(e.getMessage());
			}
		} else {
			if (quelle) {
				throw new Fehler("Es besteht bereits eine Senderanmeldung für Objekt " + objekt
						+ " und Datenspezifikation " + desc);
			}
			anmeldung.add();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void connectionClosed(final ClientDavInterface connection) {
		BestaetigterDavSender.SENDER_OBJEKTE.remove(connection);
	}

	/** {@inheritDoc} */
	@Override
	public void dataRequest(final SystemObject objekt, final DataDescription desc, final byte status) {
		synchronized (anmeldungen) {
			final SenderAnmeldung neueAnmeldung = new SenderAnmeldung(objekt, desc, false);
			final SenderAnmeldung anmeldung = anmeldungen.get(neueAnmeldung);
			if (anmeldung != null) {
				anmeldung.setStatus(status);
			}
		}
	}

	/**
	 * ermittelt, ob für das übergebene Objekt und die Datenbeschreibung eine
	 * Anmeldung als Quelle erfolgt ist.
	 *
	 * @param objekt
	 *            das Objekt
	 * @param dataDesc
	 *            die Datenbeschreibung
	 * @return true, wenn eine Quellenanmeldung vorliegt
	 */
	public boolean isQuelle(final SystemObject objekt, final DataDescription dataDesc) {
		boolean result = false;
		final SenderAnmeldung anmeldung = anmeldungen.get(new SenderAnmeldung(objekt, dataDesc, true));
		if (anmeldung != null) {
			result = anmeldung.isQuelle();
		}
		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Sendesteuerung wird für alle Anmeldungen unterstützt.
	 *
	 * @see ClientSenderInterface#isRequestSupported(SystemObject,
	 *      DataDescription)
	 */
	@Override
	public boolean isRequestSupported(final SystemObject objekt, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * blockiert einen Thread, bis die beschriebene Anmeldung bestätigt wurde.
	 * <br>
	 * Die einfache Implementierung pollt zyklisch den Status der Anmeldung
	 * entweer bis dieser als bestätigt gilt oder der übergebene Timeoutwert
	 * erreicht wurde.<br>
	 * Wird für den Timeoutwert ein Wert kleiner oder gleich 0 angegeben, wird
	 * unendlich lange gewartet.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param desc
	 *            die Datenbeschreibung
	 * @param timeout
	 *            die maximale Wartezeit in Sekunden
	 * @throws SendSubscriptionNotConfirmed
	 *             die Sendeanmeldung konnte innerhalb der geforderten Zeit
	 *             nicht bestätigt werden
	 */
	public synchronized void warteAufBestaetigung(final SystemObject objekt, final DataDescription desc,
			final long timeout) throws SendSubscriptionNotConfirmed {
		final long now = System.currentTimeMillis();
		final SenderAnmeldung anmeldung = anmeldungen.get(new SenderAnmeldung(objekt, desc, false));
		if (anmeldung != null) {
			if (!anmeldung.isQuelle()) {
				while (anmeldung.getStatus() != ClientSenderInterface.START_SENDING) {
					if ((timeout > 0) && ((System.currentTimeMillis() - now) > timeout)) {
						throw new SendSubscriptionNotConfirmed(
								"Sendeanmeldung innerhalb der geforderten Zeit nicht bestätigt.");
					}
					verbindung.sleep(100L);
				}
			}
		} else {
			throw new DataNotSubscribedException("Keine Anmeldung für " + objekt + ", " + desc + " erfolgt.");
		}
	}
}
