/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.hysterese;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Die Klasse beschreibt eine Stufe innerhalb eines Hystereseobjekts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class HystereseStufe implements Struktur, Kopierbar {

	/**
	 * der Name der Hysteresstufe.
	 */
	private String name;

	/**
	 * der Anfangswert des Bereiches der Stufe.
	 */
	private Number anfangsWert;

	/**
	 * der Endwert des Bereiches der Stufe.
	 */
	private Number endWert;

	/**
	 * Konstruktor mit Angabe des Namens und des Anangs- und Endwertes.
	 *
	 * @param name
	 *            der Name der Stufe
	 * @param anfang
	 *            der Anfangswert
	 * @param ende
	 *            der Endwert
	 */
	public HystereseStufe(final String name, final Number anfang, final Number ende) {
		this.name = name;
		this.anfangsWert = anfang;
		this.endWert = ende;
	}

	/**
	 * ermittelt, ob ein Wert in der Hysteresestufe enthalten ist.
	 *
	 * @param wert
	 *            der Wert
	 * @return <i>true</i>, wenn der Wert enthalten ist
	 */
	boolean entHaelt(final double wert) {
		boolean result = false;
		if (wert >= anfangsWert.doubleValue()) {
			if (wert <= endWert.doubleValue()) {
				result = true;
			}
		}
		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public HystereseStufe erzeugeKopie() {
		return new HystereseStufe(name, anfangsWert, endWert);
	}

	/**
	 * liefert den Anfangswert der Hysteresestufe.
	 *
	 * @return anfangsWert
	 */
	public Number getAnfangsWert() {
		return anfangsWert;
	}

	/**
	 * liefert den Endwert der Hysteresestufe.
	 *
	 * @return endWert
	 */
	public Number getEndWert() {
		return endWert;
	}

	/**
	 * liefert den Name der Stufe.
	 *
	 * @return name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * <i>Struktur</i>, um den lesenden Zugriff auf Elemente des Objekts über
	 * einen Namen zu ermöglichen.<br>
	 * Folgende Zugriffe sind möglich:
	 * <ul>
	 * <li>stufe - liefert den Namen der Stufe</li>
	 * <li>anfangswert - liefert den Anfangswert der Stufe</li>
	 * <li>endwert - liefert den Endwert der Stufe</li>
	 * </ul>
	 *
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String bezeichner) {
		Object result = null;
		if ("stufe".equals(bezeichner)) {
			result = name;
		} else if ("anfangswert".equals(bezeichner)) {
			result = anfangsWert;
		} else if ("endwert".equals(bezeichner)) {
			result = endWert;
		}
		return result;
	}

	/**
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * <i>Struktur</i>, um den schreibenden Zugriff auf Elemente des Objekts
	 * über einen Namen zu ermöglichen.<br>
	 * Folgende Zugriffe sind möglich:
	 * <ul>
	 * <li>stufe - setzt den Namen der Stufe</li>
	 * <li>anfangswert - setzt den Anfangswert der Stufe</li>
	 * <li>endwert - setzt den Endwert der Stufe</li>
	 * </ul>
	 * Die Ãbergabe der Daten muss passend für den Typ des Elements erfolgen.
	 * Passen Zieltyp und übergebener Typ nicht zusammen, wird ein
	 * <i>ArgumentFehler</i> geworfen.
	 *
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String attribut, final Object daten) {
		try {
			if ("stufe".equals(attribut)) {
				name = daten.toString();
			} else if ("anfangswert".equals(attribut)) {
				anfangsWert = (Number) daten;
			} else if ("endwert".equals(attribut)) {
				endWert = (Number) daten;
			} else {
				throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT);
			}
		} catch (@SuppressWarnings("unused") final ClassCastException e) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}
	}

	/**
	 * Die Funktion liefert eine Zeichenkette, die die Hysteresestufe
	 * repräsentiert.<br>
	 * Die Ausgabe erfolgt im Format "&lt;Name der
	 * Stufe&gt;,"&lt;Anfangswert&gt;,"&lt;Endwert&gt;". {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return name + "," + anfangsWert + "," + endWert;
	}

	/**
	 * ermittelt, ob mit dem übergebenen Wert der Bereich der Hysteresestufe
	 * nach oben verlassen wird.
	 *
	 * @param wert
	 *            der neue Wert
	 * @return true, wenn der Bereich verlassen wird
	 */
	boolean ueberSchritten(final double wert) {
		boolean result = false;
		if (wert > endWert.doubleValue()) {
			result = true;
		}
		return result;
	}

	/**
	 * ermittelt, ob mit dem übergebenen Wert der Bereich der Hysteresestufe
	 * nach unten verlassen wird.
	 *
	 * @param wert
	 *            der neue Wert
	 * @return true, wenn der Bereich verlassen wird
	 */
	boolean unterSchritten(final double wert) {
		boolean result = false;
		if (wert < anfangsWert.doubleValue()) {
			result = true;
		}
		return result;
	}
}
