/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.trend;

import java.util.Date;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * Testklasse zum Test der Klasse: {@link TrendObjekt}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TrendObjektTest {

	/**
	 * erzeugen eines Trendobjekts.
	 */
	@Test
	public void testTrendObjekt() {
		TrendObjekt trend = new TrendObjekt(null, 0);
		Assert.assertEquals(NichtWert.NICHTWERT, trend.getBezug());

		final String bezugTest = "Der Bezug";
		trend = new TrendObjekt(bezugTest, 0);
		Assert.assertEquals(bezugTest, trend.getBezug());
	}

	/**
	 * hinzufügen eines Werts.
	 */
	@Test
	public void testAddWert() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(12, null);
		Assert.assertTrue(12 == ((Number) trend.getWert()).longValue());
		trend.addWert(13, null);
		Assert.assertTrue(13 == ((Number) trend.getWert()).longValue());
		trend.addWert(14, null);
		Assert.assertTrue(14 == ((Number) trend.getWert()).longValue());

		final Zeitstempel zs = (Zeitstempel) trend.getZeit();
		zs.add(-1);
		try {
			trend.addWert(15, zs);
			Assert.fail("Vergangener Wert sollte abgewiesen werden");
		} catch (final ArgumentFehler e) {
			// Fehler korrekt ausgelöst
		}
	}

	/**
	 * hinzufügen eines Wertes bei begrenzter Kapazität.
	 */
	@Test
	public void testAddWertKapazitaet() {
		final TrendObjekt trend = new TrendObjekt(null, 3);
		trend.addWert(12, null);
		Assert.assertTrue(1 == (trend.getAnzahl()).longValue());
		trend.addWert(12, null);
		Assert.assertTrue(2 == (trend.getAnzahl()).longValue());
		trend.addWert(12, null);
		Assert.assertTrue(3 == (trend.getAnzahl()).longValue());
		trend.addWert(12, null);
		Assert.assertTrue(3 == (trend.getAnzahl()).longValue());
		trend.addWert(12, null);
		Assert.assertTrue(3 == (trend.getAnzahl()).longValue());
	}

	/**
	 * Auslesen des letzten Wertes.
	 */
	@Test
	public void testGetWert() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		Assert.assertEquals(NichtWert.NICHTWERT, trend.getWert());
		trend.addWert(222, null);
		Assert.assertEquals(222, ((Number) trend.getWert()).intValue());
	}

	/**
	 * Auslesen des Zeitpunktes des letzten Wertes.
	 */
	@Test
	public void testGetZeit() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		Assert.assertEquals(NichtWert.NICHTWERT, trend.getZeit());

		final Zeitstempel zs = new Zeitstempel();
		trend.addWert(222, zs);
		Assert.assertEquals(zs, trend.getZeit());
	}

	/**
	 * Maximale Anzahl der konstanten Werte.
	 */
	@Test
	public void testGetMaxKonstant() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(12, null);
		trend.addWert(13, null);
		trend.addWert(12, null);
		trend.addWert(12, null);
		trend.addWert(12, null);
		Assert.assertEquals(3, trend.getMaxKonstant().intValue());
	}

	/**
	 * Maximale Anzahl der fallenden Werte.
	 */
	@Test
	public void testGetMaxFallend() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(13, null);
		trend.addWert(13, null);
		trend.addWert(13, null);
		trend.addWert(12, null);
		trend.addWert(11, null);
		Assert.assertEquals(3, trend.getMaxFallend().intValue());
	}

	/**
	 * Maximale Anzahl der steigenden Werte.
	 */
	@Test
	public void testGetMaxSteigend() {
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(12, null);
		trend.addWert(12, null);
		trend.addWert(12, null);
		trend.addWert(13, null);
		trend.addWert(14, null);
		Assert.assertEquals(3, trend.getMaxSteigend().intValue());
	}

	/**
	 * ermitteln, ob die Daten im angegebenen Bereich fallend sind.
	 */
	@Test
	public void testIsFallend() {
		final long now = new Date().getTime();
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(13, new Zeitstempel(now - 6));
		trend.addWert(13, new Zeitstempel(now - 5));
		trend.addWert(13, new Zeitstempel(now - 4));
		trend.addWert(12, new Zeitstempel(now - 3));
		trend.addWert(11, new Zeitstempel(now - 2));

		Assert.assertTrue("Trend ist nicht fallend", trend.isFallend(5, null)
				.getBoolWert());
		Assert.assertTrue("Trend ist nicht fallend", trend.isFallend(new ZeitDauer(5),
				new Zeitstempel(now - 1)).getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isFallend(6, null).getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isFallend(new ZeitDauer(6),
				new Zeitstempel(now - 2)).getBoolWert());

		trend.addWert(12, new Zeitstempel(now - 1));
		Assert.assertFalse("Trend ist fallend", trend.isFallend(5, null).getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isFallend(new ZeitDauer(5),
				new Zeitstempel(now - 1)).getBoolWert());
	}

	/**
	 * ermitteln, ob die Daten im angegebenen Bereich steigend sind.
	 */
	@Test
	public void testIsSteigend() {
		final long now = new Date().getTime();
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(11, new Zeitstempel(now - 6));
		trend.addWert(12, new Zeitstempel(now - 5));
		trend.addWert(13, new Zeitstempel(now - 4));
		trend.addWert(13, new Zeitstempel(now - 3));
		trend.addWert(13, new Zeitstempel(now - 2));

		Assert.assertTrue("Trend ist nicht steigend", trend.isSteigend(null, null)
				.getBoolWert());
		Assert.assertTrue("Trend ist nicht steigend", trend.isSteigend(5, null)
				.getBoolWert());
		Assert.assertTrue("Trend ist nicht fallend",
				trend.isSteigend(new ZeitDauer(5), new Zeitstempel(now - 1)).getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isSteigend(6, null)
				.getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isSteigend(new ZeitDauer(6),
				new Zeitstempel(now - 2)).getBoolWert());

		trend.addWert(12, new Zeitstempel(now - 1));
		Assert.assertFalse("Trend ist fallend", trend.isSteigend(5, null)
				.getBoolWert());
		Assert.assertFalse("Trend ist fallend", trend.isSteigend(new ZeitDauer(5),
				new Zeitstempel(now - 1)).getBoolWert());
	}

	/**
	 * ermitteln, ob die Daten im angegebenen Bereich konstant sind.
	 */
	@Test
	public void testIsKonstant() {
		final long now = new Date().getTime();
		final TrendObjekt trend = new TrendObjekt(null, 0);
		trend.addWert(11, new Zeitstempel(now - 6));
		trend.addWert(11, new Zeitstempel(now - 5));
		trend.addWert(11, new Zeitstempel(now - 4));
		trend.addWert(11, new Zeitstempel(now - 3));
		trend.addWert(11, new Zeitstempel(now - 2));

		Assert.assertTrue("Trend ist nicht konstant", trend.isKonstant(5, null)
				.getBoolWert());
		Assert.assertTrue("Trend ist nicht fallend",
				trend.isKonstant(new ZeitDauer(5), new Zeitstempel(now - 1)).getBoolWert());
		Assert.assertFalse("Trend ist konstant", trend.isKonstant(6, null)
				.getBoolWert());
		Assert.assertFalse("Trend ist konstant", trend.isKonstant(new ZeitDauer(6),
				new Zeitstempel(now - 2)).getBoolWert());

		trend.addWert(12, new Zeitstempel(now - 1));
		Assert.assertFalse("Trend ist konstant", trend.isKonstant(5, null)
				.getBoolWert());
		Assert.assertFalse("Trend ist konstant", trend.isKonstant(new ZeitDauer(5),
				new Zeitstempel(now - 1)).getBoolWert());
	}
}
