/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import java.util.Calendar;
import java.util.GregorianCalendar;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;

/**
 * Testklasse zum Test der Klasse: {@link Zeitstempel}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitstempelTest {

	/**
	 * Test der gültigen Konstruktoren für einen Zeitstempel.
	 */
	@Test
	public void testZeitstempelOK() {
		new Zeitstempel();
		new Zeitstempel("24.12.2006 12:13:14,15");
		new Zeitstempel("24.12.2006 12:13:14");
		new Zeitstempel("24.12.2006 12:13");
		new Zeitstempel("24.12.2006 12");
		new Zeitstempel("24.12.2006");
		new Zeitstempel("12:13:14,150");
		new Zeitstempel("12:13:14");
		new Zeitstempel("12:13");
		new Zeitstempel("12");
	}

	/**
	 * Test der ungültigen Konstruktoren für einen Zeitstempel.
	 */
	@Test
	public void testZeitstempelFehler() {
		final String[] zeitAngaben = { "12.2. 12:13:14,15",
				"12.2.100 12:13:14,15", "12:13:14:15", "12:13:14,1599",
				"30.2.2006 12:13:14,15", "10.13.2006 12:13:14,15",
				"10.2.0000 12:13:14,15", "10.2.2006 24:13:14,15",
				"10.2.2006 12:73:14,15", "10.2.2006 12:13:84,15" };

		for (final String angabe : zeitAngaben) {
			try {
				new Zeitstempel(angabe);
				Assert.fail("Ungültige Zeitangabe " + angabe + " akzeptiert");
			} catch (final ArgumentFehler e) {
				// erwartetet Exception
			}
		}
	}

	/**
	 * Test des Konstruktors mit Angabe der Zeit in Millisekunden.
	 */
	@Test
	public void testZeitstempelLong() {
		final long jetzt = System.currentTimeMillis();
		final Zeitstempel zs = new Zeitstempel(jetzt);
		Assert.assertEquals(jetzt, zs.getMilliSekunden());
	}

	/**
	 * Setzen des Zeitstempels per Millisekundenwert.
	 */
	@Test
	public void testSetMilliSekunden() {
		final long jetzt = System.currentTimeMillis();
		final Zeitstempel zs = new Zeitstempel();
		zs.setMilliSekunden(jetzt);
		Assert.assertEquals(jetzt, zs.getMilliSekunden());
	}

	/**
	 * Auslesen der Strukturelemente der Klasse.
	 */
	@Test
	public void testGetStrukturElement() {
		final GregorianCalendar cal = new GregorianCalendar();
		final Zeitstempel zs = new Zeitstempel(cal.getTimeInMillis());
		Assert.assertTrue(cal.get(Calendar.DAY_OF_MONTH) == (Long) zs.getStrukturElement("tag"));
		Assert.assertTrue((cal.get(Calendar.MONTH) + 1) == (Long) zs.getStrukturElement("monat"));
		Assert.assertTrue(cal.get(Calendar.YEAR) == (Long) zs.getStrukturElement("jahr"));
		Assert.assertTrue(cal.get(Calendar.HOUR_OF_DAY) == (Long) zs.getStrukturElement("stunde"));
		Assert.assertTrue(cal.get(Calendar.MINUTE) == (Long) zs.getStrukturElement("minute"));
		Assert.assertTrue(cal.get(Calendar.SECOND) == (Long) zs.getStrukturElement("sekunde"));
		Assert.assertTrue(cal.get(Calendar.MILLISECOND) == (Long) zs.getStrukturElement("millisekunde"));
		Assert.assertTrue(cal.getTimeInMillis() == (Long) zs.getStrukturElement("millisekunden"));
	}

	/**
	 * Auslesen eines ungültigen Strukturelements.
	 */
	@Test(expected = ObjektZugriffsFehler.class)
	public void testGetStrukturElementFehler() {
		final Zeitstempel zs = new Zeitstempel();
		zs.getStrukturElement("Tag");
	}

	/**
	 * Setzen der Strukturelemente.
	 */
	@Test
	public void testSetStrukturElement() {
		final Zeitstempel zs = new Zeitstempel();
		zs.setStrukturElement("tag", 24);
		zs.setStrukturElement("monat", 12);
		zs.setStrukturElement("jahr", 2006);
		zs.setStrukturElement("stunde", 12);
		zs.setStrukturElement("minute", 13);
		zs.setStrukturElement("sekunde", 14);
		zs.setStrukturElement("millisekunde", 150);

		Assert.assertEquals(24L, zs.getStrukturElement("tag"));
		Assert.assertEquals(12L, zs.getStrukturElement("monat"));
		Assert.assertEquals(2006L, zs.getStrukturElement("jahr"));
		Assert.assertEquals(12L, zs.getStrukturElement("stunde"));
		Assert.assertEquals(13L, zs.getStrukturElement("minute"));
		Assert.assertEquals(14L, zs.getStrukturElement("sekunde"));
		Assert.assertEquals(150L, zs.getStrukturElement("millisekunde"));

		final long now = System.currentTimeMillis();
		zs.setStrukturElement("millisekunden", now);
		Assert.assertEquals(now, zs.getStrukturElement("millisekunden"));
	}

	/**
	 * Umwandlung des Zeitstempels in eine Zeichenkette.
	 */
	@Test
	public void testToString() {
		final Zeitstempel zs = new Zeitstempel("28.2.06 12:13:14,15");
		Assert.assertEquals("28.02.2006 12:13:14,150", zs.toString());
	}

	/**
	 * Formatierte Ausgabe als Zeichenkette.
	 */
	@Test
	public void testFormat() {
		final Zeitstempel zs = new Zeitstempel("28.2.06 12:13:14,15");
		Assert.assertEquals("28.02.06 12:13:14,150", zs
				.format("TT.MM.JJ hh:mm:ss,ttt"));
		Assert.assertEquals("28.02.2006 12:13:14,150", zs.format("willi"));
		Assert.assertEquals("02.2006 12:13", zs.format("MM.JJJJ hh:mm"));
	}

	/**
	 * Verschieben des Zeitstempels über Zeitdifferenzen.
	 */
	@Test
	public void testAdd() {
		Zeitstempel zs = new Zeitstempel("28.2.06");
		zs.add(3600L * 24L * 1000L);
		Assert.assertEquals(1L, zs.getStrukturElement("tag"));
		Assert.assertEquals(3L, zs.getStrukturElement("monat"));
		Assert.assertEquals(2006L, zs.getStrukturElement("jahr"));
		Assert.assertEquals(0L, zs.getStrukturElement("stunde"));
		Assert.assertEquals(0L, zs.getStrukturElement("minute"));
		Assert.assertEquals(0L, zs.getStrukturElement("sekunde"));
		Assert.assertEquals(0L, zs.getStrukturElement("millisekunde"));

		zs = new Zeitstempel("28.2.06 10:00");
		zs.add((90L * 60L * 1000L) - 1);
		Assert.assertEquals(28L, zs.getStrukturElement("tag"));
		Assert.assertEquals(2L, zs.getStrukturElement("monat"));
		Assert.assertEquals(2006L, zs.getStrukturElement("jahr"));
		Assert.assertEquals(11L, zs.getStrukturElement("stunde"));
		Assert.assertEquals(29L, zs.getStrukturElement("minute"));
		Assert.assertEquals(59L, zs.getStrukturElement("sekunde"));
		Assert.assertEquals(999L, zs.getStrukturElement("millisekunde"));
	}

	/**
	 * Vergleich von Zeitstempeln.
	 */
	@Test
	public void testCompareTo() {
		final long now = System.currentTimeMillis();
		final Zeitstempel zs1 = new Zeitstempel(now);
		final Zeitstempel zs2 = new Zeitstempel(now);

		Assert.assertTrue(0 == zs1.compareTo(zs2));

		zs2.add(1000L);
		Assert.assertTrue(zs1.compareTo(zs2) < 0);
		Assert.assertTrue(zs2.compareTo(zs1) > 0);
	}
}
